parseURL = require("parse-url")
expect   = require("./util/expect")

###
# Constructor that will normalize a request object.
#
# @param {Object} req
# @constructor
###
class Request
	constructor: (req)->
		return new Request(req) unless this instanceof Request
		expect(req?, "Could not create Request with an empty object.")
		expect(req.url?, "Could not create a Request without a URL.")
		@req = req
		@body        = req.body ? {}
		@headers     = req.headers ? {}
		@method      = (req.method ? "GET").toUpperCase()

		if process.browser
			@ip       = "127.0.0.1"
			@referrer = window.document.referrer
			defaults  = window.location
		else
			@ip = @headers["x-forwarded-for"]?.split(",")[0] ? req.connection?.remoteAddress
			@referrer = @headers["referrer"]
			[@hostname, @port] = (@headers["x-forwarded-host"] or @headers["host"] or "").split(":")
			@protocol = if req.socket?.encrypted then "https:" else "http:"
			defaults = this

		@[key] = val for key, val of parseURL(req.url, defaults)

	###
	# Trigger another request that will redirect the browser and send a location change on the server.
	# If "back" is the url then it will attempt to redirect the user back a url, otherwise the alt will be used.
	#
	# @param {String} url
	# @param {String} alt
	###
	redirect: (url="back", alt="/")->
		url = @referrer or alt if url is "back"
		@redirectURL = url

module.exports = Request
