should  = require("should")
Request = require("../src/request.coffee")

describe "Request", ->
	it "should require a request object", ->
		Request.should.throw("Could not create Request with an empty object.")
		Request.bind(null, {}).should.throw("Could not create a Request without a URL.")
		Request({ url: "/test" }).should.be.instanceOf(Request)

	it "should use get as the default method", ->
		Request(url: "/test").should
			.have.property("method", "GET")

		Request(method: "POST", url: "/test").should
			.have.property("method", "POST")

	it "should accept headers", ->
		Request(url: "/test", headers: test: true).should
			.have.property("headers")
			.with.property("test", true)

	describe "browser", ->
		before ->
			process.browser = true
			global.window = { location, document } = 
				location:
					protocol: "http:"
					hostname: "localhost"
					port:     "3002"
				document:
					referrer: "http://google.ca"

		after -> process.browser = global.window = undefined

		it "should default to the host, port and protocol of window.location", ->
			Request(url: "/test").should
				.have.properties(
					ip:       "127.0.0.1"
					referrer: "http://google.ca"
					hostname: "localhost"
					port:     "3002"
					protocol: "http"
				)

	describe "server", ->
		it "should default to the host, port and protocol of the request", ->
			req =
				url: "/test"
				connection:
					remoteAddress: "127.0.0.1"
				headers:
					host: "localhost:3002"
					referrer: "http://google.ca"
				socket: 
					encrypted: true

			Request(req).should
				.have.properties(
					ip:       "127.0.0.1"
					referrer: "http://google.ca"
					hostname: "localhost"
					port:     "3002"
					protocol: "https"
				)